#include "include/common.h"

static char	*admin_sql[] = {
	"KILL ",
	"CREATE DATABASE ",
	"DROP DATABASE ",
	"ALTER USER ",
	"CREATE USER ",
	"DROP USER "
};

static int	admin_sql_len[] = {
	5, 16, 14, 11, 12, 10
};

static int	admin_sql_cnt = 6;

int admin_process( struct thread_t *self, int db_open_res )
{
	char			*query;
	int			i;

	extern regex_t		reg_admin_dump;
	extern regex_t		reg_admin_kill;
	extern regex_t		reg_admin_createdb;
	extern regex_t		reg_admin_dropdb;
	extern regex_t		reg_admin_createuser;
	extern regex_t		reg_admin_dropuser;
	extern regex_t		reg_admin_show;

	regmatch_t		match[5];
	int			match_res;

	char			*value;
	struct db_user_t	user_data;

	char			*sql;
	int			process_res;

	if ( self->request.data_len <= 0)
		return SQLITE_ERROR_ADMIN;

	query = self->request.data;

#ifdef __DEBUG
	printf("***** Admin extended SQL: %s\n", query);
#endif

	for ( i=admin_sql_cnt; i--; )
	{
		if ( strncasecmp( admin_sql[i], query, admin_sql_len[i] ) == 0 )
		{
			switch (i)
			{
				case 0:
					//KILL SESSION_NO
					match_res = regexec( &reg_admin_kill, query, 2, match, 0 );
					if ( match_res == 0 )
					{
					}
					//send error and return
					return response_error( self, RESPONSE_ERR_INTERNAL, "\"KILL\" SQL is not support yet." );
				break;
				case 1:
					//CREATE DATABASE DBNAME
					match_res = regexec( &reg_admin_createdb, query, 2, match, 0 );
					if ( match_res == 0 )
					{
						value = &(query[match[1].rm_so]);
						query[match[1].rm_eo] = '\0';
						if ( db_open_init( &self->db, value, NULL ) == SQLITE_OK )
						{
							memcpy( self->db.db_name, value, self->request.header.db_name_len );
							//send ok and return
							return response_info( self, RESPONSE_INFO_DB_CREATE, NULL );
						}
					}
					//send error and return
					return response_error( self, RESPONSE_ERR_DB_CREATE, NULL );
				break;
				case 2:
					//DROP DATABASE DBNAME
					match_res = regexec( &reg_admin_dropdb, query, 2, match, 0 );
					if ( match_res == 0 )
					{
						value = &(query[match[1].rm_so]);
						query[match[1].rm_eo] = '\0';
						if ( unlink( value ) == 0 )
						{
							//send ok and return
							return response_info( self, RESPONSE_INFO_DB_DROP, NULL );
						}
					}
					//send error and return
					return response_error( self, RESPONSE_ERR_DB_DROP, NULL );
				break;
				case 3:
					//ALTER USER
					if ( match_res == 0 )
					{
					}
					//send error and return
					return response_error( self, RESPONSE_ERR_INTERNAL, "\"ALTER USER\" SQL is not support yet." );
				break;
				case 4:
					//CREATE USER [NO]SUPERUSER DBNAME ID PASSWORD
					match_res = regexec( &reg_admin_createuser, query, 5, match, 0 );
					if ( match_res == 0 )
					{
						value = &(query[match[1].rm_so]);
						query[match[1].rm_eo] = '\0';
						user_data.super_user = (strncasecmp(value, "SUPERUSER", 9) == 0)?1:0;

						user_data.db_name = &(query[match[2].rm_so]);
						query[match[2].rm_eo] = '\0';

						user_data.username = &(query[match[3].rm_so]);
						query[match[3].rm_eo] = '\0';

						user_data.password = &(query[match[4].rm_so]);
						query[match[4].rm_eo] = '\0';

						if ( admin_create_user( &user_data ) == SQLITE_OK )
						{
							//send ok and return;
							return response_info( self, RESPONSE_INFO_USER_CREATE, NULL );
						}
					}
					//send error and return
					return response_error( self, RESPONSE_ERR_USER_CREATE, NULL );
				break;
				case 5:
					//DROP USER ID [DBNAME]
					match_res = regexec( &reg_admin_dropuser, query, 3, match, 0 );
					if ( match_res == 0 )
					{
						memset( &user_data, 0, sizeof(user_data) );

						user_data.username = &(query[match[1].rm_so]);
						query[match[1].rm_eo] = '\0';

						if ( match[2].rm_so > 0 )
						{
							user_data.db_name = &(query[match[2].rm_so]);
							query[match[2].rm_eo] = '\0';
						}

						if ( admin_drop_user( &user_data ) == SQLITE_OK )
						{
							//send ok and return;
							return response_info( self, RESPONSE_INFO_USER_DROP, NULL );
						}
					}
					//send error and return
					return response_error( self, RESPONSE_ERR_USER_DROP, NULL );
				break;
			}
		}
	}

	return SQLITE_ERROR_ADMIN;
}

int admin_create_user( struct db_user_t *user_data )
{
	return admin_user_manage( user_data, ADMIN_USER_MANAGE_CREATE );
}

int admin_drop_user( struct db_user_t *user_data )
{
	if ( user_data->db_name != NULL )
		return admin_user_manage( user_data, ADMIN_USER_MANAGE_DROP2 );
	else
		return admin_user_manage( user_data, ADMIN_USER_MANAGE_DROP1 );
}

int admin_user_manage( struct db_user_t *user_data, int method )
{
	extern struct db_t	db_auth;
	extern pthread_mutex_t	db_auth_mutex;

	char			*sql;
	int			result;

	switch ( method )
	{
		case ADMIN_USER_MANAGE_CREATE:
			sql = db_printf( ADMIN_SQL_CREATE_USER, user_data->username, user_data->password, user_data->super_user, user_data->db_name );
		break;
		case ADMIN_USER_MANAGE_DROP1:
			sql = db_printf( ADMIN_SQL_DROP_USER1, user_data->username );
		break;
		case ADMIN_USER_MANAGE_DROP2:
			sql = db_printf( ADMIN_SQL_DROP_USER2, user_data->username, user_data->db_name );
		break;
	}

	if ( sql == NULL )
		return SQLITE_NOMEM;

	pthread_mutex_lock( &db_auth_mutex );
	result = db_throw( &db_auth, sql );
	pthread_mutex_unlock( &db_auth_mutex );
	db_free( sql );

	return result;
}

